-- License CC0 (Creative Commons license) (c) darkfrei, 2022

love.window.setMode(1280, 800)
Width, Height = love.graphics.getDimensions( )

local function getPointBetween (p1, p2)
	local x1, y1 = p1.x, p1.y
	local x2, y2 = p2.x, p2.y
	local x = x1 + 0.5*(x2 - x1)
	local y = y1 + 0.5*(y2 - y1)
	-- from lightning:
	local t = (math.random()-0.5)
	x = x - t*(y2 - y1)/2
	y = y + t*(x2 - x1)/2
	return {x=x, y=y}
end

local function newLaser (x1,y1, x2, y2, sd)
	-- d is amount of subdivisions, for example 3
	sd = sd or 3
	
	local p1 = {x=x1, y=y1}
	local p2 = {x=x2, y=y2}
	local points = {p1, p2}
	for i = 1, sd do
		local n = (#points-1)
		local temp = {unpack (points)}
--		print ('i'..i..' n'..n)
		for j = 1, n do
			
			local point = getPointBetween (points[j], points[j+1])
			table.insert (temp, j*2,point)
--			print ('inserted', #points, j*2)
		end
		points = temp
		-- make the line smooth
		if i > sd-4 then
			local s = 0.4
			for i=2, #points-1 do
				local p1 = points[i-1]
				local p2 = points[i]
				local p3 = points[i+1]
				p2.x = 0.5*s*p1.x+(1-s)*p2.x+0.5*s*p3.x
				p2.y = 0.5*s*p1.y+(1-s)*p2.y+0.5*s*p3.y
			end
			for i=#points-1, 2, -2 do
				local p1 = points[i-1]
				local p2 = points[i]
				local p3 = points[i+1]
				p2.x = 0.5*s*p1.x+(1-s)*p2.x+0.5*s*p3.x
				p2.y = 0.5*s*p1.y+(1-s)*p2.y+0.5*s*p3.y
			end
		end
	end
--	print ('#points', #points)
	local line = {}
	

	
	for i, point in ipairs (points) do
--		print (point.x, point.y)
		table.insert (line, point.x)
		table.insert (line, point.y)
	end
	local laser = {points=points, line=line, sd = sd}
	return laser
end



function love.load()
	Laser = newLaser (100,100, 1200, 600, 6)
end

 
function love.update(dt)

end



function love.draw()
	love.graphics.setColor (1,1,1)
	love.graphics.line (Laser.line)
	love.graphics.print ('press space to change the line', 20, 20)
	
	for i = 1, #Laser.line-1, 2 do
		local t = (i-1)/(#Laser.line-2) + 0.2
		local r = -2*t*t+3*t
		local g = -16/3*t*t*t + 8*t*t -5/3*t
		local b = 2*t*t - t
		love.graphics.setColor (r,g,b)
		love.graphics.circle ('fill', Laser.line[i], Laser.line[i+1], 4)
	end
end

function love.keypressed(key, scancode, isrepeat)
	if false then
	elseif key == "space" then
		Laser = newLaser (100,100, 1200, 600, 6)
	elseif key == "escape" then
		love.event.quit()
	end
end

function love.mousepressed( x, y, button, istouch, presses )
	if button == 1 then -- left mouse button
	elseif button == 2 then -- right mouse button
	end
end

function love.mousemoved( x, y, dx, dy, istouch )
end

function love.mousereleased( x, y, button, istouch, presses )
	if button == 1 then -- left mouse button
	elseif button == 2 then -- right mouse button
	end
end